<?php

/**
 * @file
 * Contains Drupal\vardumper\Plugin\Devel\Dumper\VarDumper.
 */

namespace Drupal\vardumper\Plugin\Devel\Dumper;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\devel\DevelDumperBase;
use Drupal\vardumper\VarDumper\Dumper\HtmlDrupalDumper;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\VarDumper\Cloner\VarCloner;
use Symfony\Component\VarDumper\Dumper\CliDumper;

/**
 * Class VarDumper
 *
 * @DevelDumper(
 *   id = "vardumper",
 *   label = @Translation("Symfony var-dumper"),
 *   description = @Translation("Wrapper for Symfony var-dumper debugging tool."),
 * )
 *
 */
class VarDumper extends DevelDumperBase implements ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\vardumper\VarDumper\Dumper\HtmlDrupalDumper
   */
  protected $dumper;

  /**
   * VarDumper constructor.
   *
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   * @param \Drupal\vardumper\VarDumper\Dumper\HtmlDrupalDumper $dumper
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, HtmlDrupalDumper $dumper) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->dumper = $dumper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('vardumper_html_drupal_dumper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function dump($input, $name = NULL) {
    $cloner = $this->getVarCloner();
    $dumper = 'cli' === PHP_SAPI ? new CliDumper() : $this->dumper;

    $dumper->dump($cloner->cloneVar($input));
  }

  /**
   * {@inheritdoc}
   */
  public function export($input, $name = NULL) {
    $cloner = $this->getVarCloner();
    $dumper = 'cli' === PHP_SAPI ? new CliDumper() : $this->dumper;

    $output = fopen('php://memory', 'r+b');
    $dumper->dump($cloner->cloneVar($input), $output);
    $output = stream_get_contents($output, -1, 0);

    return $this->setSafeMarkup($output);
  }

  /**
   * {@inheritdoc}
   */
  public static function checkRequirements() {
    return class_exists('Symfony\Component\VarDumper\Cloner\VarCloner', TRUE);
  }

  /**
   * @return \Symfony\Component\VarDumper\Cloner\VarCloner
   */
  private function getVarCloner() {
    $cloner = new VarCloner();
    $myCasters = [
      'Drupal\Core\Session\UserSession' => 'Drupal\devel_vardumper\Caster\DrupalCaster::castUser',
    ];
    $cloner->addCasters($myCasters);
    return $cloner;
  }

}
